<?php

/**
 * Controller for handling the invoice section in the admin control panel.
 */
class Waindigo_Invoices_ControllerAdmin_Invoice extends XenForo_ControllerAdmin_Abstract
{

    protected $_invoiceDataWriter = 'Waindigo_Invoices_DataWriter_Invoice';

    /**
     * Section splash page.
     *
     * @return XenForo_ControllerResponse_Abstract
     */
    public function actionIndex()
    {
        $invoiceModel = $this->_getInvoiceModel();
        $recurringInvoiceModel = $this->_getRecurringInvoiceModel();

        $visitor = XenForo_Visitor::getInstance();

        $viewParams = array(
            'canManageInvoices' => $visitor->hasAdminPermission('waindigoInvoices'),
            'invoices' => array(
                'total' => $invoiceModel->countInvoices()
            ),
            'recurringInvoices' => array(
                'total' => $recurringInvoiceModel->countInvoices()
            )
        );

        return $this->responseView('Waindigo_Invoices_ViewAdmin_Splash', 'waindigo_splash_invoices', $viewParams);
    } /* END actionIndex */

    /**
     * Search for invoices.
     *
     * @return XenForo_ControllerResponse_Abstract
     */
    public function actionSearch()
    {
        $viewParams = array();

        return $this->_getInvoiceSearchResponse($viewParams);
    } /* END actionSearch */

    protected function _getInvoiceSearchResponse($viewParams = array())
    {
        return $this->responseView('Waindigo_Invoices_ViewAdmin_Invoice_Search', 'waindigo_invoice_search_invoices',
            $viewParams);
    } /* END _getInvoiceSearchResponse */

    /**
     * List of invoices.
     *
     * @return XenForo_ControllerResponse_Abstract
     */
    public function actionList()
    {
        $invoiceModel = $this->_getInvoiceModel();

        $visitor = XenForo_Visitor::getInstance();

        $page = $this->_input->filterSingle('page', XenForo_Input::UINT);
        $invoicesPerPage = 20;

        $conditions = array();

        $criteria = $this->_input->filterSingle('criteria', XenForo_Input::ARRAY_SIMPLE);
        $criteria = $this->_filterInvoiceSearchCriteria($criteria);

        $filter = $this->_input->filterSingle('_filter', XenForo_Input::ARRAY_SIMPLE);

        if ($filter && isset($filter['value'])) {
            $criteria['to_username'] = array(
                $filter['value'],
                empty($filter['prefix']) ? 'lr' : 'r'
            );
            $filterView = true;
        } else {
            $filterView = false;
        }

        $fetchOptions = array(
            'perPage' => $invoicesPerPage,
            'page' => $page,
            'order' => 'invoice_id'
        );

        $criteriaPrepared = $this->_prepareInvoiceSearchCriteria($criteria);

        $totalInvoices = $invoiceModel->countInvoices($criteriaPrepared);
        if (!$totalInvoices) {
            return $this->_getNoInvoicesError();
        }

        $invoices = $invoiceModel->getInvoices($criteriaPrepared, $fetchOptions);

        $invoices = $invoiceModel->prepareInvoices($invoices);

        $viewParams = array(
            'totalInvoices' => $totalInvoices,
            'page' => $page,
            'invoicesPerPage' => $invoicesPerPage,
            'invoices' => $invoices,

            'filterView' => $filterView,
            'filterMore' => ($filterView && $totalInvoices > $invoicesPerPage)
        );

        return $this->_getInvoiceIndexResponse($viewParams);
    } /* END actionList */

    protected function _filterInvoiceSearchCriteria(array $criteria)
    {
        foreach ($criteria as $key => $value) {
            if ($value === '') {
                unset($criteria[$key]);
            }
        }

        return $criteria;
    } /* END _filterInvoiceSearchCriteria */

    protected function _prepareInvoiceSearchCriteria(array $criteria)
    {
        foreach (array(
            'usernames',
            'to_usernames'
        ) as $field) {
            if (isset($criteria[$field]) && is_string($criteria[$field])) {
                $criteria[$field] = explode(',', $criteria[$field]);
                foreach ($criteria[$field] as &$value) {
                    $value = trim($value);
                }
                $criteria[$field] = array_filter($criteria[$field]);
            }
        }

        return $criteria;
    } /* END _prepareInvoiceSearchCriteria */

    protected function _getNoInvoicesError()
    {
        return $this->responseError(new XenForo_Phrase('waindigo_there_are_no_invoices_invoices'));
    } /* END _getNoInvoicesError */

    protected function _getInvoiceIndexResponse($viewParams = array())
    {
        return $this->responseView('Waindigo_Invoices_ViewAdmin_Invoice_List', 'waindigo_invoice_list_invoices',
            $viewParams);
    } /* END _getInvoiceIndexResponse */

    public function actionSearchFieldName()
    {
        $q = $this->_input->filterSingle('q', XenForo_Input::STRING);

        if ($q !== '') {
            $invoiceItemFieldNames = $this->_getInvoiceItemFieldModel()->getInvoiceItemFieldNames(
                array(
                    'field_name' => array(
                        $q,
                        'r'
                    )
                ), array(
                    'limit' => 10
                ));
        } else {
            $invoiceItemFieldNames = array();
        }
        $viewParams = array(
            'invoiceItemFieldNames' => $invoiceItemFieldNames
        );

        return $this->responseView('Waindigo_Invoices_ViewAdmin_Invoice_SearchFieldName', '', $viewParams);
    } /* END actionSearchFieldName */

    public function actionView()
    {
        $input = $this->_input->filter(array(
            'invoice_id' => XenForo_Input::UINT
        ));

        $invoice = $this->_getInvoiceOrError($input['invoice_id']);

        $invoiceModel = $this->_getInvoiceModel();
        $invoice = $invoiceModel->prepareInvoice($invoice);

        $this->_routeMatch->setResponseType('raw');

        $viewParams = array(
            'pdf' => $this->_getInvoiceModel()->getInvoicePdf($invoice)
        );

        return $this->responseView('Waindigo_Invoices_ViewAdmin_Invoice_View', '', $viewParams);
    } /* END actionView */

    /**
     * Helper to get the invoice choice form controller response.
     *
     * @param array $choice
     *
     * @return XenForo_ControllerResponse_Abstract
     */
    protected function _getInvoiceChoiceResponse(array $choice = array())
    {
        $viewParams = array(
            'username' => (isset($choice['username']) ? $choice['username'] : ''),
            'to_username' => (isset($choice['to_username']) ? $choice['to_username'] : '')
        );

        return $this->responseView('Waindigo_Invoices_ViewAdmin_Invoice_Choice', 'waindigo_invoice_choice_invoices',
            $viewParams);
    } /* END _getInvoiceChoiceResponse */

    protected function _getInvoiceChoiceRedirect(array $choice = array())
    {
        return $this->responseRedirect(XenForo_ControllerResponse_Redirect::RESOURCE_CANONICAL_PERMANENT,
            XenForo_Link::buildAdminLink('invoices/add', '', $choice));
    } /* END _getInvoiceChoiceRedirect */

    /**
     * Helper to get the invoice add/edit form controller response.
     *
     * @param array $invoice
     *
     * @return XenForo_ControllerResponse_Abstract
     */
    protected function _getInvoiceAddEditResponse(array $invoice = array())
    {
        $invoiceModel = $this->_getInvoiceModel();

        $viewParams = array(
            'invoice' => $invoice
        );

        return $this->responseView('Waindigo_Invoices_ViewAdmin_Invoice_Edit', 'waindigo_invoice_edit_invoices',
            $viewParams);
    } /* END _getInvoiceAddEditResponse */

    /**
     * Form to add an invoice.
     *
     * @return XenForo_ControllerResponse_Abstract
     */
    public function actionAdd()
    {
        $input = $this->_input->filter(
            array(
                'user_id' => XenForo_Input::UINT,
                'to_user_id' => XenForo_Input::UINT,
                'username' => XenForo_Input::STRING,
                'to_username' => XenForo_Input::STRING
            ));

        /* @var $userModel XenForo_Model_User */
        $userModel = XenForo_Model::create('XenForo_Model_User');

        if ((!$input['username'] && !$input['user_id']) || (!$input['to_username'] && !$input['to_user_id'])) {
            $choice = array(
                'username' => $input['username'],
                'to_username' => $input['to_username']
            );
            if (!$choice['username'] && $input['user_id']) {
                $choice['username'] = $userModel->getUserById($input['user_id']);
            }
            if (!$choice['to_username'] && $input['to_user_id']) {
                $choice['to_username'] = $userModel->getUserById($input['to_user_id']);
            }
            return $this->_getInvoiceChoiceResponse($choice);
        } else
            if (!$input['user_id'] || !$input['to_user_id']) {
                $choice = array(
                    'user_id' => $input['user_id'],
                    'to_user_id' => $input['to_user_id']
                );
                if (!$choice['user_id']) {
                    $user = $userModel->getUserByName($input['username']);
                    $choice['user_id'] = $userModel->getUserIdFromUser($user);
                }
                if (!$choice['to_user_id']) {
                    $toUser = $userModel->getUserByName($input['to_username']);
                    $choice['to_user_id'] = $userModel->getUserIdFromUser($toUser);
                }
                return $this->_getInvoiceChoiceRedirect($choice);
            }

        $invoice = array(
            'invoiceDate' => time(),
            'open' => true,
            'address' => $this->_getInvoiceModel()->getMostRecentAddressForUserId($input['user_id']),
            'to_address' => $this->_getInvoiceModel()->getMostRecentAddressForUserId($input['to_user_id']),
            'name' => $this->_getInvoiceModel()->getMostRecentNameForUserId($input['user_id']),
            'to_name' => $this->_getInvoiceModel()->getMostRecentNameForUserId($input['to_user_id'])
        );

        $invoice['invoice_number'] = $this->_getInvoiceModel()->getNextInvoiceNumberForUserId($input['user_id']);
        $invoice['user'] = $userModel->getUserById($input['user_id']);
        $invoice['to_user'] = $userModel->getUserById($input['to_user_id']);
        $invoice['paid_state'] = 'unpaid';

        return $this->_getInvoiceAddEditResponse($invoice);
    } /* END actionAdd */

    /**
     * Form to edit a specified invoice.
     *
     * @return XenForo_ControllerResponse_Abstract
     */
    public function actionEdit()
    {
        $input = $this->_input->filter(array(
            'invoice_id' => XenForo_Input::UINT
        ));

        $invoice = $this->_getInvoiceOrError($input['invoice_id']);

        $invoiceModel = $this->_getInvoiceModel();
        $invoice = $invoiceModel->prepareInvoice($invoice);

        return $this->_getInvoiceAddEditResponse($invoice);
    } /* END actionEdit */

    /**
     * Saves an invoice.
     * This may either be an insert or an update.
     *
     * @return XenForo_ControllerResponse
     */
    public function actionSave()
    {
        $this->_assertPostOnly();

        XenForo_Db::beginTransaction();

        $input = $this->_getInvoiceData();

        /* @var $userModel XenForo_Model_User */
        $userModel = XenForo_Model::create('XenForo_Model_User');

        $user = $userModel->getUserById($input['user_id']);
        $toUser = $userModel->getUserById($input['to_user_id']);

        $input['username'] = $user['username'];
        $input['to_username'] = $toUser['username'];

        $writer = XenForo_DataWriter::create($this->_invoiceDataWriter);
        if ($input['invoice_id']) {
            $writer->setExistingData($input['invoice_id']);
        }
        unset($input['invoice_id']);
        $writer->bulkSet($input);

        $invoiceItems = $this->_input->filterSingle('invoice_item', XenForo_Input::ARRAY_SIMPLE);
        $invoiceItemFields = $this->_input->filterSingle('invoice_item_field', XenForo_Input::ARRAY_SIMPLE);

        $writer->setInvoiceItems($invoiceItems, $invoiceItemFields);

        $writer->save();

        XenForo_Db::commit();

        return $this->_getInvoiceIndexRedirect($writer->get('invoice_id'));
    } /* END actionSave */

    protected function _getInvoiceData()
    {
        $inputFields = $this->_getDefaultInvoiceInputFields();

        $inputFields['paid_state'] = XenForo_Input::STRING;

        return $this->_input->filter($inputFields);
    } /* END _getInvoiceData */

    protected function _getDefaultInvoiceInputFields()
    {
        return array(
            'invoice_id' => XenForo_Input::UINT,
            'invoice_number' => XenForo_Input::UINT,
            'user_id' => XenForo_Input::UINT,
            'name' => XenForo_Input::STRING,
            'address' => XenForo_Input::STRING,
            'to_user_id' => XenForo_Input::UINT,
            'to_name' => XenForo_Input::STRING,
            'to_address' => XenForo_Input::STRING,
            'invoice_date' => XenForo_Input::STRING
        );
    } /* END _getDefaultInvoiceInputFields */

    protected function _getInvoiceIndexRedirect($invoiceId = null)
    {
        return $this->responseRedirect(XenForo_ControllerResponse_Redirect::SUCCESS,
            XenForo_Link::buildAdminLink('invoices/list') . ($invoiceId ? $this->getLastHash($invoiceId) : ''));
    } /* END _getInvoiceIndexRedirect */

    /**
     * Deletes the specified invoice.
     *
     * @return XenForo_ControllerResponse_Abstract
     */
    public function actionDelete()
    {
        $invoiceId = $this->_input->filterSingle('invoice_id', XenForo_Input::UINT);
        $invoice = $this->_getInvoiceOrError($invoiceId);

        $writer = XenForo_DataWriter::create($this->_invoiceDataWriter, XenForo_DataWriter::ERROR_EXCEPTION);
        $writer->setExistingData($invoice);

        if ($this->isConfirmedPost()) {
            $writer->set('invoice_state', 'deleted');
            $writer->save();
            $writer->delete();
            return $this->_getInvoiceIndexRedirect($writer->get('invoice_id'));
        } else { // show confirmation dialog
            $viewParams = array(
                'invoice' => $invoice
            );

            return $this->responseView('Waindigo_Invoices_ViewAdmin_Invoice_Delete', 'waindigo_invoice_delete_invoices',
                $viewParams);
        }
    } /* END actionDelete */

    /**
     * Gets the specified customer or throws an exception.
     *
     * @param integer $invoiceId
     *
     * @return array
     */
    protected function _getInvoiceOrError($invoiceId)
    {
        $info = $this->_getInvoiceModel()->getInvoiceById($invoiceId);
        if (!$info) {
            throw $this->responseException(
                $this->responseError(new XenForo_Phrase('waindigo_invoicing_requested_invoice_not_found_invoices'), 404));
        }

        return $info;
    } /* END _getInvoiceOrError */

    /**
     * Validate a single field
     *
     * @return XenForo_ControllerResponse_View XenForo_ControllerResponse_Error
     */
    public function actionValidateField()
    {
        $this->_assertPostOnly();

        $field = $this->_getFieldValidationInputParams();
        $existingKey = $this->_input->filterSingle('invoice_id', XenForo_Input::UINT);

        if ($field['name'] == "from_customer_name" || $field['name'] == "to_customer_name" ||
             $field['name'] = "new_invoice_id") {
            return $this->responseRedirect(XenForo_ControllerResponse_Redirect::SUCCESS, '',
                new XenForo_Phrase('redirect_field_validated',
                    array(
                        'name' => $field['name'],
                        'value' => $field['value']
                    )));
        }

        // handle normal fields
        return $this->_validateField($this->_invoiceDataWriter,
            array(
                'existingDataKey' => $existingKey
            ));
    } /* END actionValidateField */

    /**
     *
     * @return Waindigo_Invoices_Model_Invoice
     */
    protected function _getInvoiceModel()
    {
        return $this->getModelFromCache('Waindigo_Invoices_Model_Invoice');
    } /* END _getInvoiceModel */

    /**
     *
     * @return Waindigo_Invoices_Model_Recurring_Invoice
     */
    protected function _getRecurringInvoiceModel()
    {
        return $this->getModelFromCache('Waindigo_Invoices_Model_Recurring_Invoice');
    } /* END _getRecurringInvoiceModel */

    /**
     *
     * @return Waindigo_Invoices_Model_Invoice_Item_Field
     */
    protected function _getInvoiceItemFieldModel()
    {
        return $this->getModelFromCache('Waindigo_Invoices_Model_Invoice_Item_Field');
    } /* END _getInvoiceItemFieldModel */
}