<?php

/**
 * Model for invoices.
 */
class Waindigo_Invoices_Model_Invoice extends XenForo_Model
{

    /**
     * Constants to allow joins to extra tables in certain queries
     *
     * @var integer Join user table
     */
    const FETCH_INVOICE_ITEMS = 0x02;

    const FETCH_INVOICE_ITEM_FIELDS = 0x04; // do not use
    const FETCH_INVOICE_ITEMS_AND_FIELDS = 0x06;

    /**
     *
     * @var string Invoice table name
     */
    protected $_tableName = 'invoice_waindigo';

    /**
     * Gets invoice that matches the given conditions.
     *
     * @param array $conditions Conditions to apply to the fetching
     * @param array $fetchOptions Collection of options that relate to fetching
     *
     * @return array false
     */
    public function getInvoice(array $conditions = array(), array $fetchOptions = array())
    {
        $fetchOptions['limit'] = 1;

        $whereClause = $this->prepareInvoiceConditions($conditions, $fetchOptions);
        $sqlClauses = $this->prepareInvoiceFetchOptions($fetchOptions);
        $limitOptions = $this->prepareLimitFetchOptions($fetchOptions);

        return $this->_getDb()->fetchRow(
            $this->limitQueryResults(
                '
				SELECT invoice.*
					' . $sqlClauses['selectFields'] . '
				FROM ' . $this->_tableName . ' AS invoice
				' . $sqlClauses['joinTables'] . '
				WHERE ' . $whereClause . '
			', $limitOptions['limit'], $limitOptions['offset']));
    } /* END getInvoice */

    /**
     * Gets the specified invoice if it exists.
     *
     * @param string $invoiceId
     *
     * @return array false
     */
    public function getInvoiceById($id)
    {
        $fetchOptions = array();
        $conditions = array(
            'invoice_id' => $id
        );
        return $this->getInvoice($conditions, $fetchOptions);
    } /* END getInvoiceById */

    /**
     * Gets invoices that match the given conditions.
     *
     * @param array $conditions Conditions to apply to the fetching
     * @param array $fetchOptions Collection of options that relate to fetching
     *
     * @return array Format: [invoice id] => info
     */
    public function getInvoices(array $conditions = array(), array $fetchOptions = array())
    {
        $whereClause = $this->prepareInvoiceConditions($conditions, $fetchOptions);
        $sqlClauses = $this->prepareInvoiceFetchOptions($fetchOptions);
        $limitOptions = $this->prepareLimitFetchOptions($fetchOptions);

        return $this->fetchAllKeyed(
            $this->limitQueryResults(
                '
				SELECT invoice.*
				' . $sqlClauses['selectFields'] . '
				FROM ' . $this->_tableName . ' AS invoice
				' . $sqlClauses['joinTables'] . '
				WHERE ' . $whereClause . '
				' . $sqlClauses['orderClause'] . '
			', $limitOptions['limit'],
                $limitOptions['offset']), 'invoice_id');
    } /* END getInvoices */

    /**
     * Gets the count of invoices that match the specified conditions.
     *
     * @param array $conditions
     *
     * @return integer
     */
    public function countInvoices(array $conditions = array(), array $fetchOptions = array())
    {
        $whereClause = $this->prepareInvoiceConditions($conditions, $fetchOptions);
        $sqlClauses = $this->prepareInvoiceFetchOptions($fetchOptions);

        return $this->_getDb()->fetchOne(
            '
			SELECT COUNT(*)
			FROM ' . $this->_tableName . ' AS invoice
			' . $sqlClauses['joinTables'] . '
			WHERE ' . $whereClause);
    } /* END countInvoices */

    /**
     * Prepares join-related fetch options.
     *
     * @param array $fetchOptions
     *
     * @return array Containing 'selectFields' and 'joinTables' keys.
     */
    public function prepareInvoiceFetchOptions(array $fetchOptions)
    {
        $sharedFetchOptions = $this->prepareInvoiceSharedFetchOptions($fetchOptions);
        $selectFields = $sharedFetchOptions['selectFields'];
        $joinTables = $sharedFetchOptions['joinTables'];
        $orderBy = $sharedFetchOptions['orderBy'];

        if (!empty($fetchOptions['join'])) {
            if ($fetchOptions['join'] & self::FETCH_INVOICE_ITEMS) {
                $selectFields .= ',
					invoice_item.*';
                $joinTables .= '
					LEFT JOIN invoice_item_waindigo ON (invoice.invoice_id = invoice_item.invoice_id)';
            }
            if ($fetchOptions['join'] & self::FETCH_INVOICE_ITEMS_AND_FIELDS) {
                $selectFields .= ',
					invoice_item_field.*';
                $joinTables .= '
					LEFT JOIN invoice_item_field_waindigo ON (invoice_item.invoice_item_id = invoice_item_field.invoice_item_id)';
            }
        }

        return array(
            'selectFields' => $selectFields,
            'joinTables' => $joinTables,
            'orderClause' => ($orderBy ? "ORDER BY $orderBy" : '')
        );
    } /* END prepareInvoiceFetchOptions */

    public function prepareInvoiceSharedFetchOptions($fetchOptions)
    {
        $selectFields = '';
        $joinTables = '';
        $orderBy = '';

        if (!empty($fetchOptions['order'])) {
            switch ($fetchOptions['order']) {
                case 'invoice_id':
                    $orderBy = 'invoice.invoice_id';
                case 'invoice_date':
                default:
                    $orderBy = 'invoice.invoice_date';
                    $secondaryOrderBy = 'invoice.invoice_id';
            }
            if (!isset($fetchOptions['orderDirection']) || $fetchOptions['orderDirection'] == 'desc') {
                $orderBy .= ' DESC';
                if ($secondaryOrderBy)
                    $secondaryOrderBy .= ' DESC';
            } else {
                $orderBy .= ' ASC';
                if ($secondaryOrderBy)
                    $secondaryOrderBy .= ' ASC';
            }
            if ($secondaryOrderBy)
                $orderBy .= ', ' . $secondaryOrderBy;
        }
        return array(
            'selectFields' => $selectFields,
            'joinTables' => $joinTables,
            'orderBy' => $orderBy
        );
    } /* END prepareInvoiceSharedFetchOptions */

    /**
     * Prepares a set of conditions to select users against.
     *
     * @param array $conditions List of conditions.
     * @param array $fetchOptions The fetch options that have been provided. May
     * be edited if criteria requires.
     *
     * @return string Criteria as SQL for where clause
     */
    public function prepareInvoiceConditions(array $conditions, array &$fetchOptions)
    {
        $db = $this->_getDb();

        $sqlConditions = $this->prepareInvoiceSharedConditions($conditions, $fetchOptions);

        return $this->getConditionsForClause($sqlConditions);
    } /* END prepareInvoiceConditions */

    /**
     * Prepares a set of conditions to select invoices or recurring invoices
     * against.
     *
     * @param array $conditions List of conditions.
     * @param array $fetchOptions The fetch options that have been provided. May
     * be edited if criteria requires.
     *
     * @return array $sqlConditions
     */
    public function prepareInvoiceSharedConditions(array $conditions, array &$fetchOptions)
    {
        $db = $this->_getDb();
        $sqlConditions = array();

        if (isset($conditions['invoice_id'])) {
            if (is_array($conditions['invoice_id'])) {
                if (!empty($conditions['invoice_id'])) {
                    $sqlConditions[] = 'invoice.invoice_id IN (' . $db->quote($conditions['invoice_id']) . ')';
                }
            } else {
                $sqlConditions[] = 'invoice.invoice_id = ' . $db->quote($conditions['invoice_id']);
            }
        }

        if (isset($conditions['company_name']) || isset($conditions['from_company_name']) ||
             isset($conditions['to_company_name'])) {
            if (isset($conditions['company_name'])) {
                if (is_array($conditions['company_name'])) {
                    $quoteLike = XenForo_Db::quoteLike($conditions['company_name'][0], $conditions['company_name'][1],
                        $db);
                    $sqlConditions[] = '(customer_to.company_name LIKE ' . $quoteLike .
                         ' OR customer_from.company_name LIKE ' . $quoteLike . ')';
                } else {
                    $sqlConditions[] = '(customer_to.company_name = ' . $db->quote($conditions['company_name']) .
                         ' OR customer_from.company_name = ' . $db->quote($conditions['company_name']) . ')';
                }
            } else {
                if (isset($conditions['from_company_name'])) {
                    $sqlConditions[] = 'customer_from.company_name = ' . $db->quote($conditions['from_company_name']);
                }
                if (isset($conditions['to_company_name'])) {
                    $sqlConditions[] = 'customer_to.company_name = ' . $db->quote($conditions['to_company_name']);
                }
            }
            if (!isset($fetchOptions['join'])) {
                $fetchOptions['join'] = self::FETCH_COMPANY_NAMES;
            } elseif (!$fetchOptions['join'] & self::FETCH_COMPANY_NAMES) {
                $fetchOptions['join'] |= self::FETCH_COMPANY_NAMES;
            }
        }

        if (isset($conditions['any_user_id'])) {
            if (is_array($conditions['any_user_id'])) {
                $sqlConditions[] = '(invoice.user_id IN (' . $db->quote($conditions['any_user_id']) . ')
    		            OR invoice.to_user_id IN (' . $db->quote($conditions['any_user_id']) . '))';
            } else {
                $sqlConditions[] = '(invoice.user_id = ' . $db->quote($conditions['any_user_id']) . '
    		            OR invoice.to_user_id = ' . $db->quote($conditions['any_user_id']) . ')';
            }
        } elseif (isset($conditions['user_id']) || isset($conditions['to_user_id'])) {
            if (isset($conditions['user_id'])) {
                $sqlConditions[] = 'invoice.user_id = ' . $db->quote($conditions['user_id']);
            }
            if (isset($conditions['to_user_id'])) {
                $sqlConditions[] = 'invoice.to_user_id = ' . $db->quote($conditions['to_user_id']);
            }
        } elseif (isset($conditions['usernames']) || isset($conditions['to_usernames'])) {
            if (isset($conditions['usernames'])) {
                $sqlConditions[] = 'invoice.username IN (' . $db->quote($conditions['usernames']) . ')';
            }
            if (isset($conditions['to_usernames'])) {
                $sqlConditions[] = 'invoice.to_username IN (' . $db->quote($conditions['to_usernames']) . ')';
            }
        } elseif (isset($conditions['username']) || isset($conditions['to_username'])) {
            if (isset($conditions['username'])) {
                $sqlConditions[] = 'invoice.username = ' . $db->quote($conditions['username']);
            }
            if (isset($conditions['to_username'])) {
                if (is_array($conditions['to_username'])) {
                    $quoteLike = XenForo_Db::quoteLike($conditions['to_username'][0], $conditions['to_username'][1],
                        $db);
                    $sqlConditions[] = '(invoice.to_username LIKE ' . $quoteLike . ')';
                } else {
                    $sqlConditions[] = '(invoice.to_username = ' . $db->quote($conditions['to_username']) . ')';
                }
            }
        }

        if (isset($conditions['invoice_date'])) {
            $sqlConditions[] = 'invoice.invoice_date = ' . $db->quote($conditions['invoice_date']);
        }

        if (isset($conditions['invoice_number'])) {
            $sqlConditions[] = 'invoice.invoice_number = ' . $db->quote($conditions['invoice_number']);
        }

        if (isset($conditions['invoice_item_field_name'])) {
            $sqlConditions[] = 'invoice_item_field.field_name = ' . $db->quote($conditions['invoice_item_field_name']);
            if (!isset($fetchOptions['join'])) {
                $fetchOptions['join'] = self::FETCH_INVOICE_ITEMS_AND_FIELDS;
            } elseif (!$fetchOptions['join'] & self::FETCH_INVOICE_ITEMS_AND_FIELDS) {
                $fetchOptions['join'] |= self::FETCH_INVOICE_ITEMS_AND_FIELDS;
            }
        }

        if (isset($conditions['invoice_item_field_value'])) {
            $sqlConditions[] = 'invoice_item_field.field_value = ' . $db->quote($conditions['invoice_item_field_value']);
            if (!isset($fetchOptions['join'])) {
                $fetchOptions['join'] = self::FETCH_INVOICE_ITEMS_AND_FIELDS;
            } elseif (!$fetchOptions['join'] & self::FETCH_INVOICE_ITEMS_AND_FIELDS) {
                $fetchOptions['join'] |= self::FETCH_INVOICE_ITEMS_AND_FIELDS;
            }
        }

        return $sqlConditions;
    } /* END prepareInvoiceSharedConditions */

    /**
     * Returns an array containing the invoice ids found from the complete
     * result given the range specified, along with the total number of invoices
     * found.
     *
     * @param integer Find invoices with invoice_id greater than...
     * @param integer Maximum invoices to return at once
     *
     * @return array
     */
    public function getInvoiceIdsInRange($start, $limit)
    {
        $db = $this->_getDb();

        return $db->fetchCol(
            $db->limit(
                '
    			SELECT invoice_id
    			FROM invoice_waindigo
    			WHERE invoice_id > ?
    			ORDER BY invoice_id
    		', $limit), $start);
    } /* END getInvoiceIdsInRange */

    /**
     * Prepares an invoice for display.
     *
     * @param array $invoice Unprepared invoice
     *
     * @return array Prepared invoice
     */
    public function prepareInvoice(array $invoice, array $invoiceItems = null)
    {
        if (!$invoiceItems) {
            $invoiceItemModel = $this->_getInvoiceItemModel();
            $invoiceItemConditions = array(
                'invoice_id' => $invoice['invoice_id']
            );
            $invoiceItems = $invoiceItemModel->getInvoiceItems($invoiceItemConditions);
            $invoiceItems = $invoiceItemModel->prepareInvoiceItems($invoiceItems);
        }

        $invoice['subtotal'] = 0;
        $invoice['vat'] = 0;
        foreach ($invoiceItems as $invoiceItem) {
            $invoice['subtotal'] += $invoiceItem['total'];
            foreach ($invoiceItem['invoice_item_fields'] as $invoiceItemField) {
                if ($invoiceItemField['field_name'] == "VAT") {
                    $vatRate = $invoiceItemField['field_value'];
                    if ($vatRate[strlen($vatRate) - 1] == "%") {
                        $vatRate = $vatRate / 100;
                    }
                    $invoice['vat'] += $invoiceItem['cost_amount'] * $invoiceItem['quantity'] * $vatRate;
                }
            }
        }
        $invoice['subtotal'] = number_format($invoice['subtotal'], 2);
        $invoice['total'] = number_format($invoice['subtotal'] + $invoice['vat'], 2);
        $invoice['amount'] = (int) $invoice['total'];

        $invoice['invoice_items'] = array_values($invoiceItems);

        if (isset($invoice['user_id'])) {
            $invoice['user'] = $this->_getUserModel()->getUserById($invoice['user_id']);
        }

        if (isset($invoice['to_user_id'])) {
            $invoice['to_user'] = $this->_getUserModel()->getUserById($invoice['to_user_id']);
        }

        $invoice['invoice_number'] = str_pad($invoice['invoice_number'], 8, "0", STR_PAD_LEFT);

        $invoice['invoiceDate'] = strtotime($invoice['invoice_date']);

        return $invoice;
    } /* END prepareInvoice */

    /**
     * Prepares a collection of invoices for display.
     *
     * @param array $invoices Unprepared invoices
     *
     * @return array Prepared invoices
     */
    public function prepareInvoices(array $invoices)
    {
        $invoiceIds = array_keys($invoices);

        // $invoiceItemModel = $this->_getInvoiceItemModel();
        // $invoiceItemConditions = array(
        // 'invoice_ids' => $invoiceIds,
        // );
        // $invoiceItems =
        // $invoiceItemModel->getInvoiceItems($invoiceItemConditions);
        // $invoiceItems =
        // $invoiceItemModel->prepareInvoiceItems($invoiceItems);

        foreach ($invoices as $invoiceId => &$invoice) {
            $invoice = $this->prepareInvoice($invoice);
        }

        return $invoices;
    } /* END prepareInvoices */

    /**
     * Gets the count of invoices that match the specified conditions.
     *
     * @param integer $userId
     *
     * @return integer
     */
    public function getNextInvoiceNumberForUserId($userId)
    {
        $lastInvoiceNumber = $this->_getDb()->fetchOne(
            $this->limitQueryResults(
                '
    			SELECT invoice_number
    			FROM ' . $this->_tableName . ' AS invoice
    			WHERE user_id = ?
    			ORDER BY invoice_number DESC
    		', 1, 0), $userId);

        return str_pad($lastInvoiceNumber + 1, STR_PAD_LEFT);
    } /* END getNextInvoiceNumberForUserId */

    /**
     * Gets the most recent address used by the specified user.
     *
     * @param integer $userId
     *
     * @return string
     */
    public function getMostRecentAddressForUserId($userId)
    {
        return $this->_getDb()->fetchOne(
            $this->limitQueryResults(
                '
    			SELECT IF(user_id = ?, address, to_address) AS address
    			FROM ' . $this->_tableName . ' AS invoice
    			WHERE (user_id = ? AND address != \'\')
    	            OR (to_user_id = ? AND to_address != \'\')
    			ORDER BY invoice_id DESC
    		', 1, 0),
            array(
                $userId,
                $userId,
                $userId
            ));
    } /* END getMostRecentAddressForUserId */

    /**
     * Gets the most recent name used by the specified user.
     *
     * @param integer $userId
     *
     * @return string
     */
    public function getMostRecentNameForUserId($userId)
    {
        return $this->_getDb()->fetchOne(
            $this->limitQueryResults(
                '
    			SELECT IF(user_id = ?, name, to_name) AS name
    			FROM ' . $this->_tableName . ' AS invoice
    			WHERE (user_id = ? AND name != \'\')
    	            OR (to_user_id = ? AND to_name != \'\')
    			ORDER BY invoice_id DESC
    		', 1, 0),
            array(
                $userId,
                $userId,
                $userId
            ));
    } /* END getMostRecentNameForUserId */

    public function getInvoicePdf(array $invoice)
    {
        /* @var $pdf Waindigo_Invoices_Pdf_Invoice */
        $pdf = Waindigo_Pdf::create('Waindigo_Invoices_Pdf_Invoice');

        $pdf->draw($invoice);

        return $pdf;
    } /* END getInvoicePdf */

    /**
     * Determines if the invoice can be viewed with the given permissions.
     *
     * @param array $invoice
     * @param string $errorPhraseKey
     * @param array|null $viewingUser
     *
     * @return boolean
     */
    public function canViewInvoice(array $invoice, &$errorPhraseKey = '', array $viewingUser = null)
    {
        $this->standardizeViewingUserReference($viewingUser);

        if ($viewingUser['user_id'] &&
             ($viewingUser['user_id'] == $invoice['user_id'] || $viewingUser['user_id'] == $invoice['to_user_id'])) {
            return true;
        }
    } /* END canViewInvoice */

    /**
     *
     * @return Waindigo_Invoices_Model_Invoice_Item
     */
    protected function _getInvoiceItemModel()
    {
        return $this->getModelFromCache('Waindigo_Invoices_Model_Invoice_Item');
    } /* END _getInvoiceItemModel */

    /**
     *
     * @return XenForo_Model_User
     */
    protected function _getUserModel()
    {
        return $this->getModelFromCache('XenForo_Model_User');
    } /* END _getUserModel */
}