<?php

/**
 * Model for invoice items.
 */
class Waindigo_Invoices_Model_Invoice_Item extends XenForo_Model
{

    /**
     *
     * @var string Invoice item table name
     */
    protected $_tableName = 'invoice_item_waindigo';

    /**
     * Gets invoice item that matches the given conditions.
     *
     * @param array $conditions Conditions to apply to the fetching
     * @param array $fetchOptions Collection of options that relate to fetching
     *
     * @return array false
     */
    public function getInvoiceItem(array $conditions = array(), array $fetchOptions = array())
    {
        $fetchOptions['limit'] = 1;

        $whereClause = $this->prepareInvoiceItemConditions($conditions, $fetchOptions);
        $sqlClauses = $this->prepareInvoiceItemFetchOptions($fetchOptions);
        $limitOptions = $this->prepareLimitFetchOptions($fetchOptions);

        return $this->_getDb()->fetchRow(
            $this->limitQueryResults(
                '
				SELECT invoice_item.*
				' . $sqlClauses['selectFields'] . '
				FROM ' . $this->_tableName . ' AS invoice_item
				' . $sqlClauses['joinTables'] . '
				WHERE ' . $whereClause . '
			', $limitOptions['limit'], $limitOptions['offset']));
    } /* END getInvoiceItem */

    /**
     * Gets the specified invoice item if it exists.
     *
     * @param string $invoiceItemId
     *
     * @return array false
     */
    public function getInvoiceItemById($id)
    {
        $conditions = array(
            'invoice_item_id' => $id
        );
        return $this->getInvoiceItem($conditions);
    } /* END getInvoiceItemById */

    /**
     * Gets invoice items that match the given conditions.
     *
     * @param array $conditions Conditions to apply to the fetching
     * @param array $fetchOptions Collection of options that relate to fetching
     *
     * @return array Format: [invoice item id] => info
     */
    public function getInvoiceItems(array $conditions = array(), array $fetchOptions = array())
    {
        $whereClause = $this->prepareInvoiceItemConditions($conditions, $fetchOptions);
        $sqlClauses = $this->prepareInvoiceItemFetchOptions($fetchOptions);
        $limitOptions = $this->prepareLimitFetchOptions($fetchOptions);

        return $this->fetchAllKeyed(
            $this->limitQueryResults(
                '
				SELECT invoice_item.*
				' . $sqlClauses['selectFields'] . '
				FROM ' . $this->_tableName . ' AS invoice_item
				' . $sqlClauses['joinTables'] . '
				WHERE ' . $whereClause . '
				' . $sqlClauses['orderClause'] . '
			', $limitOptions['limit'], $limitOptions['offset']),
            'invoice_item_id');
    } /* END getInvoiceItems */

    public function prepareInvoiceItemFetchOptions(array $fetchOptions)
    {
        $selectFields = '';
        $joinTables = '';
        $orderBy = '';

        return array(
            'selectFields' => $selectFields,
            'joinTables' => $joinTables,
            'orderClause' => ($orderBy ? "ORDER BY $orderBy" : '')
        );
    } /* END prepareInvoiceItemFetchOptions */

    /**
     * Prepares a set of conditions to select invoice items against.
     *
     * @param array $conditions List of conditions.
     * @param array $fetchOptions The fetch options that have been provided. May
     * be edited if criteria requires.
     *
     * @return string Criteria as SQL for where clause
     */
    public function prepareInvoiceItemConditions(array $conditions, array &$fetchOptions)
    {
        $db = $this->_getDb();
        $sqlConditions = array();

        if (isset($conditions['invoice_id'])) {
            $sqlConditions[] = 'invoice_item.invoice_id = ' . $db->quote($conditions['invoice_id']);
        }

        if (isset($conditions['invoice_item_id'])) {
            $sqlConditions[] = 'invoice_item.invoice_item_id = ' . $db->quote($conditions['invoice_item_id']);
        }

        return $this->getConditionsForClause($sqlConditions);
    } /* END prepareInvoiceItemConditions */

    /**
     * Prepares an invoice item for display.
     *
     * @param array $invoiceItem Unprepared invoice
     *
     * @return array Prepared invoice item
     */
    public function prepareInvoiceItem(array $invoiceItem, array $invoiceItemFields = array())
    {
        $invoiceItem['invoice_item_fields'] = array_values($invoiceItemFields);

        $invoiceItem['total'] = $invoiceItem['cost_amount'] * $invoiceItem['quantity'];

        $invoiceItem['cost_amount'] = number_format($invoiceItem['cost_amount'], 2, '.', '');
        $invoiceItem['total'] = number_format($invoiceItem['total'], 2, '.', '');

        return $invoiceItem;
    } /* END prepareInvoiceItem */

    /**
     * Prepares a collection of invoice items for display.
     *
     * @param array $invoices Unprepared invoice items
     *
     * @return array Prepared invoice items
     */
    public function prepareInvoiceItems(array $invoiceItems)
    {
        $invoiceItemIds = array_keys($invoiceItems);

        $invoiceItemFieldModel = $this->_getInvoiceItemFieldModel();
        $invoiceItemFieldConditions = array(
            'invoice_item_ids' => $invoiceItemIds
        );
        $invoiceItemFields = $invoiceItemFieldModel->getInvoiceItemFields($invoiceItemFieldConditions);
        $invoiceItemFields = $invoiceItemFieldModel->prepareInvoiceItemFields($invoiceItemFields);
        foreach ($invoiceItems as $invoiceItemId => $invoiceItem) {
            $fields = array();
            if (isset($invoiceItemFields[$invoiceItemId])) {
                $fields = $invoiceItemFields[$invoiceItemId];
            }
            $invoiceItems[$invoiceItemId] = $this->prepareInvoiceItem($invoiceItem, $fields);
        }

        return $invoiceItems;
    } /* END prepareInvoiceItems */

    /**
     *
     * @return Waindigo_Invoices_Model_Invoice_Item_Field
     */
    protected function _getInvoiceItemFieldModel()
    {
        return $this->getModelFromCache('Waindigo_Invoices_Model_Invoice_Item_Field');
    } /* END _getInvoiceItemFieldModel */
}