<?php

/**
 * Model for invoice item fields.
 */
class Waindigo_Invoices_Model_Invoice_Item_Field extends XenForo_Model
{

    /**
     *
     * @var string Invoice item field table name
     */
    protected $_tableName = 'invoice_item_field_waindigo';

    /**
     * Gets invoice item field that matches the given conditions.
     *
     * @param array $conditions Conditions to apply to the fetching
     * @param array $fetchOptions Collection of options that relate to fetching
     *
     * @return array false
     */
    public function getInvoiceItemField(array $conditions = array(), array $fetchOptions = array())
    {
        $fetchOptions['limit'] = 1;

        $whereClause = $this->prepareInvoiceItemFieldConditions($conditions, $fetchOptions);
        $sqlClauses = $this->prepareInvoiceItemFieldFetchOptions($fetchOptions);
        $limitOptions = $this->prepareLimitFetchOptions($fetchOptions);

        return $this->_getDb()->fetchRow(
            $this->limitQueryResults(
                '
				SELECT invoice_item_field.*
					' . $sqlClauses['selectFields'] . '
				FROM ' . $this->_tableName . ' AS invoice_item_field
				' . $sqlClauses['joinTables'] . '
				WHERE ' . $whereClause . '
			', $limitOptions['limit'], $limitOptions['offset']));
    } /* END getInvoiceItemField */

    /**
     * Gets the specified invoice item field if it exists.
     *
     * @param string $invoiceItemFieldId
     *
     * @return array false
     */
    public function getInvoiceItemFieldById($id)
    {
        $conditions = array(
            'invoice_item_field_id' => $id
        );
        return $this->getInvoiceItemField($conditions);
    } /* END getInvoiceItemFieldById */

    /**
     * Gets invoice item fields that match the given conditions.
     *
     * @param array $conditions Conditions to apply to the fetching
     * @param array $fetchOptions Collection of options that relate to fetching
     *
     * @return array Format: [invoice item field id] => info
     */
    public function getInvoiceItemFields(array $conditions = array(), array $fetchOptions = array())
    {
        $whereClause = $this->prepareInvoiceItemFieldConditions($conditions, $fetchOptions);
        $sqlClauses = $this->prepareInvoiceItemFieldFetchOptions($fetchOptions);
        $limitOptions = $this->prepareLimitFetchOptions($fetchOptions);

        return $this->fetchAllKeyed(
            $this->limitQueryResults(
                '
				SELECT invoice_item_field.*
				' . $sqlClauses['selectFields'] . '
				FROM ' . $this->_tableName . ' AS invoice_item_field
				' . $sqlClauses['joinTables'] . '
				WHERE ' . $whereClause . '
				' . $sqlClauses['orderClause'] . '
			', $limitOptions['limit'], $limitOptions['offset']),
            'invoice_item_field_id');
    } /* END getInvoiceItemFields */

    /**
     * Gets invoice item fields that match the given conditions.
     *
     * @param array $conditions Conditions to apply to the fetching
     * @param array $fetchOptions Collection of options that relate to fetching
     *
     * @return array
     */
    public function getInvoiceItemFieldNames(array $conditions = array(), array $fetchOptions = array())
    {
        $whereClause = $this->prepareInvoiceItemFieldConditions($conditions, $fetchOptions);
        $limitOptions = $this->prepareLimitFetchOptions($fetchOptions);

        return $this->_getDb()->fetchCol(
            $this->limitQueryResults(
                '
				SELECT invoice_item_field.field_name
				FROM ' . $this->_tableName . ' AS invoice_item_field
				WHERE ' . $whereClause . '
				GROUP BY field_name
			', $limitOptions['limit'], $limitOptions['offset']));
    } /* END getInvoiceItemFieldNames */

    public function prepareInvoiceItemFieldFetchOptions(array $fetchOptions)
    {
        $selectFields = '';
        $joinTables = '';
        $orderBy = '';

        return array(
            'selectFields' => $selectFields,
            'joinTables' => $joinTables,
            'orderClause' => ($orderBy ? "ORDER BY $orderBy" : '')
        );
    } /* END prepareInvoiceItemFieldFetchOptions */

    public function prepareInvoiceItemFieldConditions(array $conditions, array &$fetchOptions)
    {
        $db = $this->_getDb();
        $sqlConditions = array();

        if (isset($conditions['invoice_item_field_id'])) {
            $sqlConditions[] = 'invoice_item_field.invoice_item_field_id = ' .
                 $db->quote($conditions['invoice_item_field_id']);
        }

        if (isset($conditions['invoice_item_id'])) {
            $sqlConditions[] = 'invoice_item_field.invoice_item_id = ' . $db->quote($conditions['invoice_item_id']);
        }

        if (isset($conditions['field_name'])) {
            if (is_array($conditions['field_name'])) {
                $sqlConditions[] = 'invoice_item_field.field_name LIKE ' .
                     XenForo_Db::quoteLike($conditions['field_name'][0], $conditions['field_name'][1], $db);
            } else {
                $sqlConditions[] = 'invoice_item_field.field_name = ' . $db->quote($conditions['field_name']);
            }
        }

        return $this->getConditionsForClause($sqlConditions);
    } /* END prepareInvoiceItemFieldConditions */

    /**
     * Prepares a collection of invoice item fields for display.
     *
     * @param array $invoiceItemFields Unprepared invoice item fields
     *
     * @return array Prepared invoice item fields
     */
    public function prepareInvoiceItemFields(array $invoiceItemFields)
    {
        $fields = array();

        foreach ($invoiceItemFields as $invoiceItemField) {
            $fields[$invoiceItemField['invoice_item_id']][] = $invoiceItemField;
        }

        return $fields;
    } /* END prepareInvoiceItemFields */

    public static function processFieldValue($field)
    {
        return $field;
    } /* END processFieldValue */
}