<?php

abstract class Waindigo_Pdf_Block
{
    /**
     * Standard approach to caching other model objects for the lifetime of the model.
     *
     * @var array
     */
    protected $_modelCache = array();

    protected $_pdf = null;
    protected $_page = null;

    /**
     * Gets the specified model object from the cache. If it does not exist,
     * it will be instantiated.
     *
     * @param string $class Name of the class to load
     *
     * @return XenForo_Model
     */
    public function getModelFromCache($class)
    {
        if (!isset($this->_modelCache[$class])) {
            $this->_modelCache[$class] = XenForo_Model::create($class);
        }

        return $this->_modelCache[$class];
    }

    /**
     * Factory method to get the named model. The class must exist or be autoloadable
     * or an exception will be thrown.
     *
     * @param string Class to load
     *
     * @return Waindigo_Pdf_Block
     */
    public static function create($class)
    {
        $createClass = XenForo_Application::resolveDynamicClass($class, 'pdfblock_waindigo');
        if (!$createClass) {
            throw new XenForo_Exception("Invalid model '$class' specified");
        }

        return new $createClass;
    }

    public function draw(Waindigo_Pdf $pdf)
    {
        $this->_pdf = $pdf;
        $this->_page = $pdf->getCurrentPage();
    }

    public function addNewPage()
    {
        $this->_page = $this->_pdf->addNewPage();
    }

    /**
     * Returns the total width in points of the string using the specified
     * font and size.
     *
     * @param string $string
     * @param Zend_Pdf_Resource_Font $font
     * @param float $fontSize Font size in points
     * @return float
     */
    public static function widthForStringUsingFontSize($string, Zend_Pdf_Resource_Font $font, $fontSize)
    {
        // remove characters not recognised by this font
        $string = $font->decodeString($font->encodeString($string, 'UTF-8'), 'UTF-8');

        // get widths of characters
        $characters = self::_convertStringToCharacterArray($string);
        $glyphs = $font->glyphNumbersForCharacters($characters);
        $widths = $font->widthsForGlyphs($glyphs);

        // get width of string
        return (array_sum($widths) / $font->getUnitsPerEm()) * $fontSize;
    }

    public static function getFontSizeHeight(Zend_Pdf_Resource_Font $font, $fontSize) {
        return ($font->getLineHeight() / $font->getUnitsPerEm()) * $fontSize;
    }

    protected static function _convertStringToCharacterArray($string)
    {
        $characters = preg_split('//u', $string, -1, PREG_SPLIT_NO_EMPTY);
        array_walk($characters, array('self','_utf8ord'));
        return $characters;
    }

    protected static function _utf8ord(&$character)
    {
        if (isset($character[0])) {
            $value = ord($character[0]);
            if ($value <= 0x7F) {
                $character = $value;
            } else if ($value < 0xC2) {
                $character = false;
            } else if ($value <= 0xDF) {
                $character = ($value & 0x1F) << 6 | (ord($character[1]) & 0x3F);
            } else if ($value <= 0xEF) {
                $character = ($value & 0x0F) << 12 | (ord($character[1]) & 0x3F) << 6
                | (ord($character[2]) & 0x3F);
            } else if ($value <= 0xF4) {
                $character = ($value & 0x0F) << 18 | (ord($character[1]) & 0x3F) << 12
                | (ord($character[2]) & 0x3F) << 6
                | (ord($character[3]) & 0x3F);
            } else {
                $character = 0;
            }
        }
    }
}